﻿using System;
using System.Drawing;
using System.Windows.Forms;
using WindowsFormsApplication1.Properties;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.CellType;
using FarPoint.Win.Spread.Model;
using Appearance = FarPoint.Win.Spread.Appearance;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            fpSpread.AllowUserFormulas = true;
            fpSpread.AllowDragFill = true;

            fpSpread.AllowSheetMove = false;
            fpSpread.AllowColumnMove = true;
            fpSpread.TabStripInsertTab = false;

            m_SheetView.AutoFilterMode = AutoFilterMode.EnhancedContextMenu;
            m_SheetView.ColumnHeader.AutoFilterIndex = 0;
            m_SheetView.ColumnHeader.AutoTextIndex = 0;
            m_SheetView.ColumnHeader.AutoText = HeaderAutoText.Letters;
            m_SheetView.ColumnHeader.AutoSortIndex = 0;
            m_SheetView.AllowGroup = true;
            m_SheetView.GroupBarInfo.Visible = true;
            m_SheetView.GroupingPolicy = GroupingPolicy.CollapseAll;
            m_SheetView.GroupBarInfo.Text = @"Ziehen Sie eine Spalte in diesen Bereich, um nach dieser zu gruppieren";
            m_SheetView.GroupBarInfo.BackColor = Color.Aqua;
            m_SheetView.Models.ColumnHeaderStyle = new HeaderSheetStyleModel(m_SheetView.Models.ColumnAxis);

            m_SheetView.ColumnHeader.RowCount = 2;

            m_SheetView.ColumnCount = 10;
            m_SheetView.RowCount = 50;

            m_SheetView.ColumnHeader.Cells[1, 0].Text = "Long Text 1";
            m_SheetView.ColumnHeader.Cells[1, 1].Text = "Long Text 2";
            m_SheetView.ColumnHeader.Cells[1, 2].Text = "Long Text 3";
            m_SheetView.ColumnHeader.Cells[1, 3].Text = "Long Text 4";
            m_SheetView.ColumnHeader.Cells[1, 4].Text = "Long Text 5";
            m_SheetView.ColumnHeader.Cells[1, 5].Text = "Long Text 6";
            m_SheetView.ColumnHeader.Cells[1, 6].Text = "Long Text 7";
            m_SheetView.ColumnHeader.Cells[1, 7].Text = "Long Text 8";
            m_SheetView.ColumnHeader.Cells[1, 8].Text = "Long Text 9";
            m_SheetView.ColumnHeader.Cells[1, 9].Text = "Long Text 10";

            #region Fake Data

            for (int i = 0; i < 50; i++)
            {
                m_SheetView.Cells[i, 0].Value = Convert.ToDecimal(_rng.Next(123456789, 987654321));
            }

            for (int i = 0; i < 50; i++)
            {
                m_SheetView.Cells[i, 1].Value = _rng.Next(123456789, 987654321);
            }

            for (int i = 0; i < 50; i++)
            {
                m_SheetView.Cells[i, 2].Value = RandomString(6);
            }

            for (int i = 0; i < 50; i++)
            {
                m_SheetView.Cells[i, 3].Value = RandomDateTime();
            }

            #endregion Fake Data
        }

        private void fpSpread_Grouped(object sender, EventArgs e)
        {
            var model = fpSpread.ActiveSheet.Models.Data as GroupDataModel;
            if (model != null)
            {
                System.Diagnostics.Stopwatch w = System.Diagnostics.Stopwatch.StartNew();

                for (int i = 0; i < model.RowCount; i++)
                {
                    if (model.IsGroup(i))
                    {
                        var grou = model.GetGroup(i);
                        grou.Text = fpSpread.ActiveSheet.ColumnHeader.Cells[1, grou.Column].Text + ":" + model.GetValue(i + 1, grou.Column);
                    }
                }

                w.Stop();
                System.Diagnostics.Debug.WriteLine(string.Format("RowCount {0}: Time: {1}", model.RowCount, w.ElapsedTicks/model.RowCount));
            }
        }

        private readonly Random _rng = new Random();

        private const string _chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

        private string RandomString(int size)
        {
            char[] buffer = new char[size];

            for (int i = 0; i < size; i++)
            {
                buffer[i] = _chars[_rng.Next(_chars.Length)];
            }
            return new string(buffer);
        }

        private DateTime RandomDateTime()
        {
            DateTime start = new DateTime(1995, 1, 1);
            Random gen = new Random();

            int range = (DateTime.Today - start).Days;
            return start.AddDays(gen.Next(range));
        }
    }

    internal class HeaderSheetStyleModel : ISheetStyleModel, IDisposable
    {
        #region Fields

        private ISheetAxisModel m_ColumnAxis;

        //private  SortHelper m_sortHelper;
        private HeaderCellTypeRenderer m_Renderer;

        public event SheetStyleModelEventHandler Changed;

        private class HeaderRenderer : HeaderCellTypeRenderer
        {
            #region Constructors

            public HeaderRenderer() : base()
            {
            }

            #endregion Constructors

            #region Methods

            protected override Rectangle PaintAdditionalSymbols(Graphics graphics, Rectangle rectangle, Appearance appearance)
            {
                var rect = rectangle;
                Point pos = new Point(rect.X + rect.Width - AdditionalIconSize, rect.Y + ((rect.Height - AdditionalIconSize + 4)/2));
                graphics.DrawImage(Resources.filter_16, pos.X, pos.Y, 12, 12);
                rect = new Rectangle(rectangle.Left, rectangle.Top, rectangle.Width - AdditionalIconSize, rectangle.Height);

                // Sortierungselement anzeigen
                if (appearance.SortState != SortState.None)
                {
                    var bounds = new Rectangle(rect.X + rect.Width - AdditionalIconSize, rect.Y, 16, 16);

                    switch (appearance.SortState)
                    {
                        case SortState.Ascending:
                            graphics.DrawImage(Resources.simple_up_button_16, bounds.X, bounds.Y, bounds.Width, bounds.Height);
                            break;
                        case SortState.Descending:
                            graphics.DrawImage(Resources.simple_down_button_16, bounds.X, bounds.Y, bounds.Width, bounds.Height);
                            break;
                    }
                    return new Rectangle(rect.Left, rect.Top, rect.Width - AdditionalIconSize, rect.Height);
                }

                return rectangle;
                //// Filterelement anzeigen
                //if (m_isHitListColumn)
                //{
                //    Point pos = new Point(rectangle.X + rectangle.Width - AdditionalIconSize, rectangle.Y + ((rectangle.Height - AdditionalIconSize) / 2));
                //    graphics.DrawImage(DDLImages.Size16Images.filter_16, pos.X, pos.Y, 16, 16);
                //    return new Rectangle(rectangle.Left, rectangle.Top, rectangle.Width - AdditionalIconSize, rectangle.Height);
                //}
            }

            #endregion Methods
        }

        #endregion Fields

        #region Properties

        public int AltRowCount
        {
            get { return 1; }
            set { throw new NotImplementedException(); }
        }

        #endregion Properties

        #region Constructors

        internal HeaderSheetStyleModel(ISheetAxisModel columnAxis) //, SortHelper sortHelper)
        {
            m_ColumnAxis = columnAxis;
            m_Renderer = new HeaderCellTypeRenderer();
        }

        #endregion Constructors

        #region Methods

        public bool IsEmpty()
        {
            return false;
        }

        public StyleInfo GetCompositeInfo(int row, int column, int altIndex, StyleInfo destInfo)
        {
            if (destInfo == null)
            {
                destInfo = new StyleInfo();
            }

            if (row == -1 || column == -1)
            {
                return destInfo; // Nur echte Einzelzellen zulassen
            }

            if (row == 0)
            {
                destInfo.Renderer = new HeaderRenderer();
            }
            else
            {
                destInfo.Renderer = m_Renderer;
                m_Renderer.IsUserColumn = column%2 == 0;
            }

            return destInfo;
        }

        public StyleInfo GetDirectInfo(int row, int column, StyleInfo destInfo)
        {
            return null;
        }

        public StyleInfo GetDirectAltRowInfo(int index, StyleInfo destInfo)
        {
            return null;
        }

        public void SetDirectInfo(int row, int column, StyleInfo info)
        {
        }

        public void SetDirectAltRowInfo(int index, StyleInfo info)
        {
        }

        public bool IsRowUsed(int row)
        {
            return true;
        }

        protected virtual void OnChanged(SheetStyleModelEventArgs e)
        {
            var handler = Changed;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        public void Dispose()
        {
            m_Renderer = null;

            if (Changed != null)
            {
                foreach (Delegate @delegate in Changed.GetInvocationList())
                {
                    Changed -= (SheetStyleModelEventHandler) @delegate;
                }
            }
        }

        #endregion Methods
    }

    internal class HeaderCellTypeRenderer : IRenderer, IDisposable
    {
        #region Fields

        public const int AdditionalIconSize = 18;

        #endregion Fields

        #region Properties

        public bool IsUserColumn { get; set; }

        #endregion Properties

        #region Constructors

        public HeaderCellTypeRenderer()
        {
        }

        #endregion Constructors

        #region Methods

        public virtual void PaintCell(Graphics graphics, Rectangle rectangle, Appearance appearance, object value, bool isSelected, bool isLocked, float zoomFactor)
        {
            var availableRect = PaintAdditionalSymbols(graphics, rectangle, appearance);

            string str = (value == null) ? null : value.ToString();
            if (str != null)
            {
                graphics.FillRectangle(Brushes.Bisque, availableRect);

                var labelBrush = Brushes.Black;

                var font = IsUserColumn ? new Font(appearance.Font, FontStyle.Bold | FontStyle.Italic) : appearance.Font;
                if (zoomFactor != 1.0)
                {
                    font = new Font(font.Name, font.Size*zoomFactor, font.Style);
                }

                graphics.DrawString(str, font, labelBrush, availableRect, StringFormat.GenericDefault);
            }
        }

        protected virtual Rectangle PaintAdditionalSymbols(Graphics graphics, Rectangle rectangle, Appearance appearance)
        {
            return rectangle;
        }

        private static StringAlignment GetHorizontalAlignment(CellHorizontalAlignment align)
        {
            switch (align)
            {
                case CellHorizontalAlignment.Left:
                    return StringAlignment.Near;
                case CellHorizontalAlignment.Right:
                    return StringAlignment.Far;
                default:
                    return StringAlignment.Center;
            }
        }

        private static StringAlignment GetVerticalAlignment(CellVerticalAlignment align)
        {
            switch (align)
            {
                case CellVerticalAlignment.Top:
                    return StringAlignment.Near;
                case CellVerticalAlignment.Bottom:
                    return StringAlignment.Far;
                default:
                    return StringAlignment.Center;
            }
        }

        public bool CanBeOverflown()
        {
            return false;
        }

        public bool CanOverflow()
        {
            return false;
        }

        public Size GetPreferredSize(Graphics g, Size size, Appearance appearance, object value, float zoomFactor)
        {
            var sizeF = g.MeasureString(value as string, appearance.Font);
            return new Size((int) Math.Ceiling(sizeF.Width), (int) Math.Ceiling(sizeF.Height));
        }

        public void Dispose()
        {
        }

        #endregion Methods
    }
}